; Read filtered Fermi-LAT data from Fermi-Science-Tools. Output the further filtered list of photon events.
; By Wenlei Chen      Oct 2013 
; Need IDL_ASTRO_LIBRARY, Fermi_Mask_Source.pro

PRO Fermi_Filter_Main

;Environment_Setting
SetEnv,'1FHL_DIR=X:\Research\Data\Fermi_LAT\1FHL\'
SetEnv,'LAT_weekly_DIR=X:\Research\Data\Fermi_LAT\Weekly\Filtered\Pass7V6_10GeV-60GeV_CLEAN\'
SetEnv,'OUTPUT_DIR=X:\Research\Data\OUTPUT\'

;Read 1FHL
dir_fhl = GetEnv('1FHL_DIR')
fhl_file_name = 'gll_psch_v07.fits'
fhl = MRDFITS(dir_fhl+fhl_file_name,1)

fhl_pos = dblarr(2,514)
fhl_pos(0,*) = fhl.(1)   ; RA
fhl_pos(1,*) = fhl.(2)   ; DEC

;Null
null_pos = dblarr(2,1)
null_pos(0,0) = 380.
null_pos(1,0) = 200.

;Read photon data and generate data array
;Search files
Read, 'Energy level?   10GeV-20GeV: 1, 20GeV-30GeV: 2, 30GeV-40GeV: 3, 40GeV-50GeV: 4, 50GeV-60GeV: 5   ', lev 
sub_dir = strcompress('level'+string(fix(lev))+'\',/remove_all)
dir_pho  = GetEnv('LAT_weekly_DIR') + sub_dir
file_pho = 'lat_w???_l?.fits'
file_arr = File_Search(dir_pho+file_pho,count=num_files)
file_arr = file_arr(Sort(file_arr))
FOR i_file = 0, num_files-1 DO BEGIN
  file_fig_tmp  = file_arr(i_file)
  file_fig_tmp  = StrMid(file_fig_tmp, StrLen(dir_pho), StrLen(file_fig_tmp)-StrLen(dir_pho))
  Print, 'file name: ', String(i_file), ', ', file_fig_tmp
ENDFOR
i_file_beg  = 0
i_file_end  = 0
print,'i_file_beg: '
Read,i_file_beg
print,'i_file_end: '
Read,i_file_end

;Creat data array
IF lev EQ 1 THEN BEGIN
  n_r = 1024*num_files
  e_range = '10GeV-20GeV'
ENDIF
IF lev EQ 2 THEN BEGIN
  n_r = 256*num_files
  e_range = '20GeV-30GeV'
ENDIF
IF lev EQ 3 THEN BEGIN
  n_r = 128*num_files
  e_range = '30GeV-40GeV'
ENDIF
IF lev EQ 4 THEN BEGIN
  n_r = 128*num_files
  e_range = '40GeV-50GeV'
ENDIF
IF lev EQ 5 THEN BEGIN
  n_r = 128*num_files
  e_range = '50GeV-60GeV'
ENDIF
data_arr = dblarr(7,n_r)

;Write data into data array
i_file = i_file_beg
n_rows = 0L
WHILE i_file LE i_file_end DO BEGIN
  file_tmp = file_arr(i_file)
  data_arr_tmp = MRDFITS(file_tmp,1)
  tmp_arr0 = data_arr_tmp.(0)
  tmp_arr1 = data_arr_tmp.(1)
  tmp_arr2 = data_arr_tmp.(2)
  tmp_arr3 = data_arr_tmp.(3)
  tmp_arr4 = data_arr_tmp.(4)
  tmp_arr5 = data_arr_tmp.(9)
  tmp_arr6 = data_arr_tmp.(10)
  FOR n_tmp = 0, n_elements(data_arr_tmp)-1 DO BEGIN
    data_arr(0,n_rows) = tmp_arr0(n_tmp)    ;ENERGY
    data_arr(1,n_rows) = tmp_arr1(n_tmp)    ;RA
    data_arr(2,n_rows) = tmp_arr2(n_tmp)    ;DEC
    data_arr(3,n_rows) = tmp_arr3(n_tmp)    ;L
    data_arr(4,n_rows) = tmp_arr4(n_tmp)    ;B
    data_arr(5,n_rows) = tmp_arr5(n_tmp)    ;TIME
    data_arr(6,n_rows) = tmp_arr6(n_tmp)    ;EVENT ID
    n_rows++
  ENDFOR
  i_file++
ENDWHILE

;Fermi Filter
;filtered_arr = Fermi_Mask_Source(data_arr, fhl_pos, n_rows)      ; Mask Sources in 1FHL
filtered_arr = Fermi_Mask_Source(data_arr, null_pos, n_rows)     ; Null

;Output the photon lists
file_beg  = file_arr(i_file_beg)
file_end  = file_arr(i_file_end)
file_beg  = StrMid(file_beg, StrLen(dir_pho), StrLen(file_beg)-StrLen(dir_pho))
file_end  = StrMid(file_end, StrLen(dir_pho), StrLen(file_end)-StrLen(dir_pho))
s_w = StrMid(file_beg, 5, 3)
e_w = StrMid(file_end, 5, 3)

;Creat data structure array
num = n_elements(filtered_arr)/7
str = {ENERGY:0.0D, RA:0.0D, DEC:0.0D, GLON:0.0D, GLAT:0.0D, TIME:0.0D, EVENT_ID:0L}
str_arr = REPLiCATE(str,num)
i = 0L
WHILE i ne num DO BEGIN
  str_arr[i].(0) = filtered_arr(0,i)
  str_arr[i].(1) = filtered_arr(1,i)
  str_arr[i].(2) = filtered_arr(2,i)
  str_arr[i].(3) = filtered_arr(3,i)
  str_arr[i].(4) = filtered_arr(4,i)
  str_arr[i].(5) = filtered_arr(5,i)
  str_arr[i].(6) = filtered_arr(6,i)
  i++
ENDWHILE
;Write fits file
f_name = strcompress(GetEnv('OUTPUT_DIR')+'LAT_filtered_PhotonList'+e_range+'_w'+s_w+'-w'+e_w+'.fits',/remove_all)
MWRFITS, str_arr, f_name

Print, 'DONE!'

END